#include <ioavr.h>
#include "compiler.h"
#include "usb_avr_registers.h"
#include "wdt_drv.h"
#include "conf_usb.h"
#include "usb_device_request.h"
#include "usb_host_drv.h"
#include "usb_host_transfer.h"
#include "usb_host_hub_drv.h"
#include "usb_host_hid_drv.h"

#include "usb_peripheral_drv.h"

#include "spi_rpc.h"

#pragma location="FWVERSION_F"
const __flash U16 fwversion=0x0101;
#pragma required=fwversion


//volatile U8 activeGuitar=0;
__no_init volatile U8 activeGuitar@0x100;
volatile U8 newReportReady=0;
U8 usb_user_process_request(U8 bmRequestType, U8 bmRequest)
{   
    if (bmRequest == GET_REPORT && bmRequestType == REQUEST_INTERFACE_CLASS_READ)
    {
        Usb_ack_receive_setup();
        Usb_write_byte(0x21);
        Usb_write_byte(0x26);
        Usb_write_byte(0x01);
        if (activeGuitar==GUITAR_GH3)    Usb_write_byte(0x06);
        else                             Usb_write_byte(0x00);

        Usb_write_byte(0x00);
        Usb_write_byte(0x00);
        Usb_write_byte(0x00);
        Usb_write_byte(0x00);

        Usb_send_control_in();                       
        while( !Is_usb_receive_out() );
        Usb_ack_receive_out();
        return TRUE;
    }
    else if (bmRequestType == REQUEST_INTERFACE_CLASS_WRITE)
    { // catches anything.. but most interestingly SET_IDLE, SET_PROTOCOL and SET_REPORT
        if (bmRequest == SET_IDLE && activeGuitar==GUITAR_GH3) return FALSE;
        Usb_ack_receive_setup();
        Usb_send_control_in();                       
        return TRUE;
        
    }
/*
    else if (bmRequest == GET_STATUS)
    {
        return FALSE;
      //TODO: We should never get here!! check it and remove..
        Usb_ack_receive_setup();
        Usb_send_control_in();
        if (bmRequest==GET_INTERFACE)
            while(!Is_usb_in_ready());
        return TRUE;
    }
*/    
    else return FALSE;    
}

U8 usb_user_get_descriptor(U8 bmRequestType, U8 descriptor_type, U8 __flash ** pbuffer, U8 * data_to_transfer)
{
   switch (descriptor_type)
   {
    case HID_REPORT_DESCRIPTOR:
        dprintf("Responding with hid report descroptor %x bytes\r",sizeof(sony_hid_report_descriptor));
        *data_to_transfer = sizeof(sony_hid_report_descriptor);
        *pbuffer = (U8 __flash *) (sony_hid_report_descriptor);
        return TRUE;
      case HID_DESCRIPTOR:
            *data_to_transfer = sizeof(S_usb_hid_descriptor)+sizeof(S_usb_hid_specific_descriptor);
            *pbuffer = (U8 code *)&desc_conf_gh3.hid;
            dprintf("Sending HID descr\r");
            return TRUE;
   default:
       return FALSE;
   }

//   return FALSE;
}

//! @brief This function configures the endpoints of the device application
//! This function is called when the set configuration request has been received.
void usb_user_endpoint_init(U8 conf_nb)
{
    if (activeGuitar!= GUITAR_NONE)
    {
        Usb_select_endpoint(GUITAR_EP_INT_IN);
        if (! usb_configure_endpoint(GUITAR_EP_INT_IN,
                             TYPE_INTERRUPT,     
                             DIRECTION_IN,  
                             SIZE_64,       
                             ONE_BANK,     
                             NYET_ENABLED))
            dprintf("Failed configuring GUITAR_EP_INT_IN");

        Usb_select_endpoint(GUITAR_EP_INT_OUT);

        if (!usb_configure_endpoint(GUITAR_EP_INT_OUT,
                             TYPE_INTERRUPT,     
                             DIRECTION_OUT,  
                             SIZE_64,       
                             ONE_BANK,     
                             NYET_ENABLED))
            dprintf("Failed configuring GUITAR_EP_INT_OUT");
            

    }
}

void Main_notify_spi_rpc_complete(U8 msgSz, U8 * msg)
{
    U8 m = *msg;

    if ((m & SPI_RPC_MSG_MSG_MASK) == SPI_RPC_MSG_SETDEVICE)
    {
        U8 c = m & SPI_RPC_MSG_DATA_MASK;

        if (c == SPI_RPC_DEVICE_GH3<<SPI_RPC_MSG_DATA_OFS)
        {
            activeGuitar=GUITAR_GH3;
        }
        else if (c == SPI_RPC_DEVICE_RB<<SPI_RPC_MSG_DATA_OFS)
        {
            activeGuitar=GUITAR_RB;
        }
        else
            dprintf(":\r");
    }
    // We _should_ check for newReportReady below as we might be in the process of sending
    // -however, we're most likely not so this will just send the latest report instead.
    else if ((m & SPI_RPC_MSG_MSG_MASK) == SPI_RPC_MSG_SETREPORT)// && !newReportReady)
    {
      
        sony_hid_report[0]=*(msg+1);
        sony_hid_report[1]=*(msg+2);
        sony_hid_report[2]=*(msg+3);
        sony_hid_report[5]=*(msg+4);
        sony_hid_report[19]=*(msg+5);
        newReportReady=1;
    }
    else
        dprintf("!");
}

#pragma inline=forced
void processGH3report(void)
{
    U8 r0 = sony_hid_report[0];
    U8 r2 = sony_hid_report[2];

    // make correct pad neutral value
    if (r2==0x08) sony_hid_report[2] = 0x0f;

    
    // add GH3 pad digital values
    if (0<r2 && r2<4) sony_hid_report[0x07] = 0xff; else sony_hid_report[0x07]=0x0;
    if (8>r2 && r2>=5) sony_hid_report[0x08] = 0xff;  else sony_hid_report[0x08]=0x0;
    if (r2<2 || r2==7) sony_hid_report[0x09]=0xff; else sony_hid_report[0x09]=0x0;
    if (2<r2 && r2<6) sony_hid_report[0x0a]=0xff; else sony_hid_report[0x0a]=0x0;    
        
    // Add GH3 fret buttons digital values
    if (r0 & 0x01) sony_hid_report[0x0e] = 0xff;else sony_hid_report[0x0e] = 0;
    if (r0 & 0x02) sony_hid_report[0x0d] = 0xff;else sony_hid_report[0x0d] = 0;
    if (r0 & 0x04) sony_hid_report[0x0c] = 0xff;else sony_hid_report[0x0c] = 0;
    if (r0 & 0x08) sony_hid_report[0x0b] = 0xff;else sony_hid_report[0x0b] = 0;
    if (r0 & 0x10) sony_hid_report[0x0f] = 0xff;else sony_hid_report[0x0f] = 0;
  
    // GH3 Tilt.
    if (r0 & 0x20)
    {
      //TODO: we might need a timer to hold this value for a moment.. the 0x20
      // goes away imediately.
        sony_hid_report[19] = 0x84;
        sony_hid_report[0] = r0 & 0x1f; // remove tilt bit for GH3
    }
    else
    {  
        sony_hid_report[19] = 0xf7;
    }
}

void waitForGuitar()
{
    U16 cnt;
    
    while(! activeGuitar)
    {
        //delay for setdevice
        for (U8 cnt2=0;cnt2<50;cnt2++)
        {
            //the loop below is 8 cycles.. -> 0.1 s delay
            for(cnt=0xffff-10000;cnt>0;cnt++) 
            {
                if (activeGuitar!=0) break;
            }
            if (activeGuitar!=0) break;
        }
    }

    for (U8 cnt2=0;cnt2<PS3_REPORT_SZ;cnt2++)
        sony_hid_report[cnt2]=0;
    for (U8 cnt2=0;cnt2<4;cnt2++)
        sony_hid_report[(PS3_REPORT_SZ-7)+cnt2*2]=0x02;
    sony_hid_report[3]=0x80;
    sony_hid_report[4]=0x80;
    
    
        if (activeGuitar==GUITAR_GH3)
        {
//            const char chr[] = "\x00\x00\x0f\x80\x80\x81\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf7\x01\x7c\x01\xff\x01\x00\x02";
            dprintf("Configuring guitar GH3\r ");
            desc_str_active[0]=descr_str_langid;
            desc_str_active[1]=descr_str_manuid_gh3;
            desc_str_active[2]=descr_str_prodid_gh3;
            desc_dev_active=&desc_dev_gh3;
            desc_conf_active=&desc_conf_gh3;
            
            sony_hid_report[6]=0x80;
            sony_hid_report[PS3_REPORT_SZ-7]=0x01;
        }
        else if (activeGuitar==GUITAR_RB)
        {
//            const char chr[] = "\x00\x00\x08\x80\x80\x7f\x7f\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x02\x00\x02\x00\x02";
            dprintf("Configuring guitar RB\r ");
            desc_str_active[0]=descr_str_langid;
            desc_str_active[1]=descr_str_manuid_rb;
            desc_str_active[2]=descr_str_prodid_rb;
            desc_dev_active=&desc_dev_rb;
            desc_conf_active=&desc_conf_rb;
            
            sony_hid_report[6]=0x7f;
        }
    

}

#pragma inline=forced
void clientSendReport()
{ 
    // Swap bit 1 and 4 (fret buttons)
    U8 cnt2=sony_hid_report[0] & 0x36; 
    if (sony_hid_report[0] & 0x01)
        cnt2|=0x08;
    if (sony_hid_report[0] & 0x08)
        cnt2|=0x01;
    sony_hid_report[0]=cnt2;
    
    
     if (activeGuitar==GUITAR_GH3)
     {
         processGH3report();
     }
     // Set tilt for RB
     else
     {
       // processRBreport:
       
       //RB wammy bar is 0x80 on idle, and drops to 0 on move.
       //GH3 moves from 0x80 - 0xff
       cnt2=sony_hid_report[5];
       if (cnt2>0x80)
          sony_hid_report[5] = (cnt2-0x81)<<1;

       // set correct pad-neutral value
       if (sony_hid_report[2]==0x0f)
         sony_hid_report[2]=0x08;

       // activate TILT       
       if (sony_hid_report[19]==0x84) 
       {
          sony_hid_report[0] |= 0x20;
       }
       sony_hid_report[19]=0;
     }

             
     for (cnt2=0;cnt2<sizeof(sony_hid_report);cnt2++)
     {
        Usb_write_byte(*(sony_hid_report+cnt2));
     }
     newReportReady=0;
     Usb_ack_fifocon();
     dprintf("Sent a report %x bytes %x\r",sizeof(sony_hid_report),sony_hid_report[0]);
}

void main(void)
{
/*    DDRA_Bit0=1;
    DDRA_Bit1=1;
    DDRD = 0xf0;
    PORTD=1;
    PORTA=0;
  */  
    Wdt_off();

    
    debug_init();
    dprintf("\r\rdebug testing ready Client side\r");
    spi_rpc_init(SPI_AS_SLAVE);
    
    __enable_interrupt();
    waitForGuitar();
    dprintf("Active Guitar: %d\r",activeGuitar);

    dprintf("Enabling usb hw\r");

    usb_peripheral_drv_init();

    while(1)
    {
        __watchdog_reset();
        
        usb_peripheral_drv_task();
        if(! Is_device_enumerated()) 
            continue;
        
        //Check if INT EP has data request
        Usb_select_endpoint(GUITAR_EP_INT_IN);  
        if (newReportReady && Is_usb_write_enabled())
            clientSendReport();
        Usb_select_endpoint(0); // stuff can happen on cntr endpoint too.
        if(Is_usb_receive_out())
          Usb_ack_receive_out();  

        Usb_select_endpoint(GUITAR_EP_INT_OUT); // stuff can happen on cntr endpoint too.
        if(Is_usb_receive_out())
        {
            Usb_ack_receive_out();                
//            PIND_Bit6=1;

        }
           
    }
    
}
