#include <ioavr.h>
#include "compiler.h"
//#include "usb.h"
#include "usb_avr_registers.h"
#include "wdt_drv.h"
#include "conf_usb.h"
#include "usb_device_request.h"
#include "usb_host_drv.h"
#include "usb_host_transfer.h"
#include "usb_host_hub_drv.h"
#include "usb_host_hid_drv.h"

#include "usb_peripheral_drv.h"

#include "spi_rpc.h"

#if (USB_HOST_FEATURE == ENABLED)

#pragma location="FWVERSION_F"
const __flash U16 fwversion=0x0101;
#pragma required=fwversion


void Host_device_disconnection_action(S_Host_usb_device * hubdev)
{
    dprintf("Device disconnected\r");
}
void Host_device_connection_action()
{
    dprintf("Device connected\r");

}
void Host_hwup_action()
{
}

void Host_device_error_action(S_Host_usb_device * hubdev)
{
    dprintf("USB HOST Error\r");
}

void Host_device_enumerated_action(S_Host_usb_device * devp)
{
    U8 ret;
    dprintf("Device enumerated\r");
 
    if (devp->bDeviceClass==HUB_CLASS && devp->device_state==DEVICE_READY)
    {
        ret = usb_host_hub_enumerated_action(devp);
        if (ret)
            dprintf("Hub initialization failed\r");
    }
    else
    if (devp->bDeviceClass==HID_CLASS && devp->device_state==DEVICE_READY)
    {
        ret = usb_host_hid_enumerated_action(devp);
        if (ret)
            dprintf("Hid initialization failed\r");
    }
}

U8 usb_str_buffer[255];
U8 * USBStringToChar(U8 * str)
{
    U8 * tgt = usb_str_buffer;
    usb_str_buffer[0]='x';usb_str_buffer[1]='\0';
    U8 len=*str;
    str++;
    if (*str != STRING_DESCRIPTOR) return 0;
    str++;
    while(len)
    {
        *(tgt++)=*str;
        str+=2;
        len-=2;
    }
    *tgt='\0';
    return usb_str_buffer;   
}

// shall return true if supported
char Host_notify_new_device(S_usb_device_descriptor * devdesc, S_Host_usb_device * dev)
{
//    U8 strings[3]={0,0,0};
//    U8 i;
    U8 * chrp;
    if (devdesc->bDescriptorType != DEVICE_DESCRIPTOR)
        return 0;
    
    dev->bDeviceClass=devdesc->bDeviceClass;

    if (devdesc->idProduct == PID_GH3)
    {
        spi_rpc_set_device(GUITAR_RB);
        dprintf("Notify guitar: GH3\r");
    }
    else if (devdesc->idProduct == PID_RB)
    {
        spi_rpc_set_device(GUITAR_GH3);
        dprintf("Notify guitar: RB\r");
    }
    else
        dprintf("Notify guitar: UNKNOWN\r");

    dprintf("Usb spec: %x\r",devdesc->bscUSB);
    
    chrp=host_get_device_std_descriptor(STRING_DESCRIPTOR,0,0,dev->ctrl_pipe_nb);
    dprintf("string0: %x %x %x %x\r"
            ,*(chrp+0)
            ,*(chrp+1)
            ,*(chrp+2)
            ,*(chrp+3)
     );
    
/*    
    dprintf("New device: vid:%x pid:%x class:%x subclass:%x proto:%x configs:%x\r",
            devdesc->idVendor,
            devdesc->idProduct,
            devdesc->bDeviceClass,
            devdesc->bDeviceSubClass,
            devdesc->bDeviceProtocol,
            devdesc->bNumConfigurations
            );
    dprintf("manufstr: %x productstr: %x, serial: %x\r",
        devdesc->iManufacturer,
        devdesc->iProduct,
        devdesc->iSerialNumber
    );
            

    strings[0]=devdesc->iManufacturer;
    strings[1]=devdesc->iProduct;
    strings[2]=devdesc->iSerialNumber;

    for (i=0;i<3;i++)
    {
        if (strings[i])
        {
             chrp=host_get_device_std_descriptor(STRING_DESCRIPTOR,strings[i],0,dev->ctrl_pipe_nb);
             if (chrp==NULL)
                 continue;
                 
            chrp=USBStringToChar(chrp);
            if (chrp)
            {
                dprintf("  %s\r",chrp);
            }
        }
    }
*/
    return 1;
}
// shall return true and configure endpoints if supported
// must call host_configure_endpoint
signed char Host_notify_new_configuration(S_usb_configuration_descriptor * confdesc, S_Host_usb_device * dev)
{
    char*p=(char*)confdesc+confdesc->bLength;
    S_usb_interface_descriptor * ifdesc;
    S_usb_hid_descriptor * hiddesc;
    S_usb_endpoint_descriptor * epdesc[2];
    S_usb_hid_specific_descriptor * hidspecific;
    char i;

    dprintf("new configuration!\r");

    if (confdesc->bDescriptorType != CONFIGURATION_DESCRIPTOR)   return -1;
/*
    dprintf("maxpower: %x  bmAttibutes: %x  interfaces: %x\r",
           confdesc->MaxPower,
           confdesc->bmAttibutes,
           confdesc->bNumInterfaces);
*/
    ifdesc= (S_usb_interface_descriptor*)p;    p+= ifdesc->bLength;
    if (ifdesc->bDescriptorType != INTERFACE_DESCRIPTOR)
        return -1;

/*
    dprintf("if: %x, eps:%x, class:%x, subclass:%x, proto:%x\r",
            ifdesc->bInterfaceNumber,
            ifdesc->bNumEndpoints,
            ifdesc->bInterfaceClass,
            ifdesc->bInterfaceSubClass,
            ifdesc->bInterfaceProtocol);
*/

    if (ifdesc->bInterfaceClass==HID_CLASS)
    {
        if (dev->bDeviceClass==0)
            dev->bDeviceClass=HID_CLASS;
        
        hiddesc=(S_usb_hid_descriptor*)p;   
        if (hiddesc->bDescriptorType != HID_DESCRIPTOR)     return -1;
/*
        dprintf("HID CLASS ver:%x, country:%x  num class descr: %x\r",
                hiddesc->bcdHID,
                hiddesc->bCountryCode,
                hiddesc->bNumDescriptors
        );
*/
        // Get hid specific descriptor types
        for (i=0;i<hiddesc->bNumDescriptors;i++)
        {
            hidspecific = (S_usb_hid_specific_descriptor*)  (p + 
                sizeof(S_usb_hid_descriptor)+i*sizeof(S_usb_hid_specific_descriptor));
/*
            dprintf("HidSpecific: type:%x  size:%x\r", 
                    hidspecific->bDescriptorType,
                    hidspecific->wDescriptorLength);
*/            
            dev->misc = (void*)hidspecific->wDescriptorLength;
        }
        p+= hiddesc->bLength;
    }
    
    if (ifdesc->bNumEndpoints>2)  return -1;

    for (i=0;i<ifdesc->bNumEndpoints;i++)
    {
        epdesc[i]=(S_usb_endpoint_descriptor*)p;   p+=epdesc[i]->bLength;
        if (epdesc[i]->bDescriptorType != ENDPOINT_DESCRIPTOR)    return -1;
/*
        dprintf("endpoint %x, attr: %x,  maxsize: %x, interval %x\r",
                epdesc[i]->bEndpointAddress,
                epdesc[i]->bmAttributes,
                epdesc[i]->wMaxPacketSize,
                epdesc[i]->bInterval);
*/
    }
    
    dprintf("Configuring %d endpoint(s)!\r",ifdesc->bNumEndpoints);    
    for (i=0;i<ifdesc->bNumEndpoints;i++)
    {
        dev->ep_pipes[i] = host_configure_endpoint(epdesc[i], dev->ep_pipes[i]);
        if (dev->ep_pipes[i]==0xff)
            return -1;
    }

    return confdesc->bConfigurationValue;
}

extern U16 getreport;
void Host_sof_action() // runs every ms
{
    getreport++;
}
void Main_notify_spi_rpc_complete(U8 msgSz, U8 * msg)
{
}


void main(void)
{
    U16 cnt=0;
    DDRA_Bit0=1;
    DDRA_Bit1=1;
    PORTA=0;
    S_Host_usb_device * devp = &Host_main_device;

    Wdt_off();

    spi_rpc_init(SPI_AS_HOST);
    
    debug_init();
    
    dprintf("\r\rdebug testing ready Host side\r");
    dprintf("Enabling hw\r");
    __enable_interrupt();

   
    usb_host_drv_init();

    while(1)
    {
        __watchdog_reset();
        cnt++;
        if (cnt>0xf000)
        {
//            PINA_Bit0=1;
            cnt=0;
        }
        usb_host_drv_task(devp);
        
        if (devp->device_state==DEVICE_READY)
        {
        
            if (devp->bDeviceClass==HUB_CLASS)
                usb_host_hub_task(devp);
            else if (devp->bDeviceClass==HID_CLASS)
            {
                usb_host_hid_task(devp);
            }
        }
        devp=devp->next;
        Host_configure_address(devp->device_addr);
    }
}


#endif
