#include <ioavr.h>
#include "compiler.h"
#include "conf_usb.h"
#include "spi_rpc.h"


// | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
//                       m   s   g
//           s   z   e
//   d t a        
// Optional data in subsequent bytes -- in which case these bits are not used


#define SPI_RPC_MSG_SZ_2  (2<<SPI_RPC_MSG_SZ_OFS)
#define SPI_RPC_MSG_SZ_3  (3<<SPI_RPC_MSG_SZ_OFS)
#define SPI_RPC_MSG_SZ_4  (4<<SPI_RPC_MSG_SZ_OFS)
#define SPI_RPC_MSG_SZ_5  (5<<SPI_RPC_MSG_SZ_OFS)


void spi_rpc_set_device(U8 device)
{
    spi_rpc_write(SPI_RPC_MSG_SETDEVICE | (device<<SPI_RPC_MSG_DATA_OFS));
 }

void spi_rpc_write_report(U8 sz, U8 * dta)
{ 
    U8 szCode;
    if (sz==5) szCode = SPI_RPC_MSG_SZ_5;
    else if (sz==4) szCode = SPI_RPC_MSG_SZ_4;
    else if (sz==3) szCode = SPI_RPC_MSG_SZ_3;
    else asm("break"); // ouppsie..

    spi_rpc_write(SPI_RPC_MSG_SETREPORT | szCode);
    do
    {
        spi_rpc_write(*(dta++));
    }   while(--sz);

}

#define SPI_SEND_BUFFER_SZ 7
U8 spi_send_buffer[SPI_SEND_BUFFER_SZ];
volatile U8 spi_send_head=0;
U8 spi_send_tail=0;
U8 spi_is_sending=0;

void spi_rpc_disable()
{
        SPCR = 0;
}

void spi_rpc_init(U8 asHost)
{ 
    if (asHost)
    {
        /* Set MOSI and SCK output, all others input */
        DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK)|(1<<DD_SS);

        /* Enable SPI, Master, set clock rate fck/16 */
        SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPIE)|(1<<SPR0);//

        PORT_SS = 1;
        
    }
    else
    {
        /* Set MISO output, all others input */
        DDR_SPI = (1<<DD_MISO);
        /* Enable SPI */
        SPCR = (1<<SPE)|(1<<SPIE);

        PORT_SS = 1;
    }

    //clear any interrupt
    SPSR;
    SPDR;
}

void spi_rpc_write(U8 c)
{
    if (spi_send_head>0)
    {
        while(spi_send_head>=SPI_SEND_BUFFER_SZ-1); // block if buffer full
        spi_send_buffer[spi_send_head++ -1]=c;
    }
    else
    {
        spi_send_head=1;
        SPDR = c;
        PORT_SS = 0;
        
    }
}

void spi_write_block(U8 c)
{
    PORT_SS=0;
    SPDR = c;
    while(!(SPSR & (1<<SPIF)))
    ;
    PORT_SS=1;
}

U8 spi_read_block()
{
    while(!(SPSR & (1<<SPIF)))
    ;
    // Return Data Register 
    return SPDR;
}


#pragma vector = SPI_STC_vect
__interrupt void spi_interrupt()
{
    if (SPCR_MSTR)
    {
        if (++spi_send_tail<spi_send_head)
            SPDR = spi_send_buffer[spi_send_tail-1];
        else
        {
            PORT_SS = 1;
            spi_send_head=0;
            spi_send_tail=0;
        }
    }
    else
    {
        U8 c = SPDR;
        spi_send_buffer[spi_send_head]=c;
        if (spi_send_head==0) // first message in packet, decode size
        {
            spi_send_tail = (c >> 3) & 0x07;
            if (spi_send_tail!=0 && (c != (SPI_RPC_MSG_SETREPORT|SPI_RPC_MSG_SZ_5) ))
            {
                dprintf(".\r");
                return;
            }
        }
        spi_send_head++;
        
        if (spi_send_head>spi_send_tail) // last byte received
        {
            Main_notify_spi_rpc_complete(spi_send_tail,spi_send_buffer);
            spi_send_head=0;
            spi_send_tail=0;
        }
    }
}

