#include <ioavr.h>
#include "compiler.h"
#include "conf_usb.h"
#include "usb.h"
#include "usb_avr_registers.h"
#include "pll_drv.h"
#include "usb_host_drv.h"
#include "usb_host_transfer.h"

#if (USB_HOST_FEATURE == ENABLED)

volatile U8 private_sof_counter=0;
volatile U16 g_usb_event=0;


void Host_main_connectReset(S_Host_usb_device * dev);
void Host_main_state_unattached(void);
U8 Host_main_state_attached(S_Host_usb_device * dev);
U8 host_determine_pipe_size(U16 size);

S_Host_usb_device Host_main_device = 
{
    DEVICE_UNATTACHED,
    0, // autoassign address
    3, // autoassign ctrl pipe
    {4,0xff}, // autoassign ep_pipes
    0,
//    0,
    NULL,
    Host_main_connectReset,
    Host_main_state_unattached,
    Host_main_state_attached,
    &Host_main_device
};
static U16  c;                // As internal host start of frame counter

S_usb_endpoint_descriptor default_ctrl_ep = {
    sizeof(S_usb_endpoint_descriptor),
    ENDPOINT_DESCRIPTOR,
    EP_CONTROL | TOKEN_SETUP,
    TYPE_CONTROL,
    8,
    0
};

static U8 free_pipes=0xff;
static U8 free_addresses=0xff;

void usb_host_drv_init()
{
#if (USE_USB_PADS_REGULATOR==ENABLE)  
   Usb_enable_regulator();
#endif    
    
   Usb_force_host_mode();
   
   Pll_start_auto();
   Wait_pll_ready();
   Usb_disable();
   Usb_enable();
   Usb_unfreeze_clock();
   Usb_attach();
   Usb_enable_uvcon_pin();
   Usb_select_host();
   Usb_disable_vbus_hw_control();   // Force Vbus generation without timeout
   Host_enable_device_disconnection_interrupt();
}  
   

void Host_main_connectReset(S_Host_usb_device * dev)
{
    
        Host_disable_device_disconnection_interrupt();
        Host_send_reset();          // First USB reset
        Usb_ack_event(EVT_HOST_SOF);
        while (Is_host_reset());    // Active wait of end of reset send
        Host_ack_reset();
        //Workaround for some bugly devices with powerless pull up
        //usually low speed where data line rise slowly and can be interpretaded as disconnection
        for(c=0;c!=0xFFFF;c++)    // Basic Timeout counter
        {
           if(Is_usb_event(EVT_HOST_SOF))   //If we detect SOF, device is still alive and connected, just clear false disconnect flag
           {
              if(Is_device_disconnection())
              {
                  Host_ack_device_connection();
                  Host_ack_device_disconnection();
                  break;
              }
           }
        }
        Host_enable_device_disconnection_interrupt();
}

void Host_main_state_unattached(void)
{
    Usb_clear_all_event();
    Usb_enable_vbus();                    // Give at least device power supply!!!
//   if(Is_usb_vbus_high())
//    { device_state=DEVICE_ATTACHED; }     // If VBUS ok goto to device connection expectation
}

U8 Host_main_state_attached(S_Host_usb_device * dev)
{
    U8 retval = DEVICE_ATTACHED;
    
    
     if (Is_device_connection())     // Device pull-up detected
     {
        Host_ack_device_connection();
        Usb_clear_all_event();                // Clear all software events

       // Now device is connected, enable disconnection interrupt
        Host_enable_device_disconnection_interrupt();
        Enable_interrupt();
       // Reset device status
        Host_enable_sof();            // Start Start Of Frame generation
        Host_enable_sof_interrupt();  // SOF will be detected under interrupt
        c = 0;
        while (c<100)               // wait 100ms before USB reset
        {
           if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }// Count Start Of frame
           if (Is_host_emergency_exit() || Is_usb_bconnection_error_interrupt()) {goto device_attached_error;}
        }
        
        dev->connectReset(dev);
      
        Host_disable_device_disconnection_interrupt();
        Host_send_reset();          // First USB reset
        Usb_ack_event(EVT_HOST_SOF);
        while (Is_host_reset());    // Active wait of end of reset send
        Host_ack_reset();
        //Workaround for some bugly devices with powerless pull up
        //usually low speed where data line rise slowly and can be interpretaded as disconnection
        for(c=0;c!=0xFFFF;c++)    // Basic Timeout counter
        {
           if(Is_usb_event(EVT_HOST_SOF))   //If we detect SOF, device is still alive and connected, just clear false disconnect flag
           {
              if(Is_device_disconnection())
              {
                  Host_ack_device_connection();
                  Host_ack_device_disconnection();
                  break;
              }
           }
        }
        Host_enable_device_disconnection_interrupt();
        
        c = 0;
        while (c<100)               // wait 100ms after USB reset
        {
           if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }// Count Start Of frame
           if (Is_host_emergency_exit() || Is_usb_bconnection_error_interrupt()) {goto device_attached_error;}
        }
        retval = DEVICE_POWERED;
        c=0;
     }
     device_attached_error:
    // Device connection error, or vbus pb -> Retry the connection process from the begining
     if( Is_usb_bconnection_error_interrupt()||Is_usb_vbus_error_interrupt()||Is_usb_vbus_low())
     {
        Usb_ack_bconnection_error_interrupt();
        Usb_enable_vbus_hw_control();
        retval=DEVICE_UNATTACHED;
        Usb_disable_vbus();
        Usb_disable_vbus_pad();
        Usb_enable_vbus_pad();
        Usb_ack_vbus_error_interrupt();
        Usb_enable_vbus();
        Usb_disable_vbus_hw_control();
        Host_disable_sof();
     }
     return retval;
}



/*
This only includes the general part of the enumeration,
specialized tasks must be called for directly connected devices,
and devices connected trough hub.
*/
static U8 * enumerate_task_chrp;
void usb_host_drv_task(S_Host_usb_device* devstruct)
{
    U8 ds = devstruct->device_state;

// we use ifs instead of switch since it's faster (switch compares on int!).
    if (ds==DEVICE_READY)
    {
        return;
    }
    else if (ds==DEVICE_UNATTACHED)
    {
        if (devstruct->handle_state_unattached)
            devstruct->handle_state_unattached();
        devstruct->device_state=DEVICE_ATTACHED;
        return;
    }
    else if (ds==DEVICE_ATTACHED) {
        devstruct->device_state = devstruct->handle_state_attached(devstruct);
    }
    else if (ds==DEVICE_POWERED) {         
        //------------------------------------------------------
        //   DEVICE_POWERED state
        //
        //   - Device connection (attach) as been detected,
        //   - Wait 100ms and configure default control pipe
        //
        //Host_device_connection_action();
        if (Is_usb_event(EVT_HOST_SOF))
        {
            Usb_ack_event(EVT_HOST_SOF);
            if (c++ >= 100)                          // Wait 100ms
            {
               default_ctrl_ep.wMaxPacketSize = 8;
               devstruct->ctrl_pipe_nb = host_configure_endpoint(&default_ctrl_ep,devstruct->ctrl_pipe_nb);
               if (devstruct->ctrl_pipe_nb != 0xff)
                   devstruct->device_state = DEVICE_DEFAULT;
               else
                   devstruct->device_state = DEVICE_ERROR;
            }
        }
    }
    else if (ds==DEVICE_DEFAULT) { // Get first device descriptor
        extern S_usb_setup_data usb_request;
        extern U8 data_stage[SIZEOF_DATA_STAGE];

        usb_request.bmRequestType = 0x80;
        usb_request.bRequest      = GET_DESCRIPTOR;
        usb_request.wValue        = DEVICE_DESCRIPTOR << 8;
        usb_request.wIndex        = 0;
        usb_request.wLength       = 64;
        usb_request.uncomplete_read = TRUE;
        c = host_send_control(data_stage,devstruct->ctrl_pipe_nb);
        
         if( CONTROL_GOOD == c)
         {
            c = 0;
            while(c<20)           // wait 20ms before USB reset (special buggly devices...)
            {
               if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }
               if (Is_host_emergency_exit() || Is_usb_bconnection_error_interrupt())  {return;}
            }
            devstruct->connectReset(devstruct);
            
            c = 0;
            while(c<200)           // wait 200ms after USB reset
            {
               if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }
               if (Is_host_emergency_exit() || Is_usb_bconnection_error_interrupt())  {break;}
            }
            
            default_ctrl_ep.wMaxPacketSize = (U16)data_stage[OFFSET_FIELD_MAXPACKETSIZE];
            host_configure_endpoint(&default_ctrl_ep, devstruct->ctrl_pipe_nb);

            c = devstruct->device_addr;
            // Give an absolute device address
             if (c==0) 
             {
                 U8 i;
                 for (i=0;i<8;i++)
                 {
                     if (free_addresses & (1<<i))
                     {
                         free_addresses &= ~(1<<i);
                         c = i+1;
                         break;
                     }
                 }
             }
             if (c==0) 
             {
                 devstruct->device_state = DEVICE_ERROR; 
                 return;
             }
            
            host_usb_device_set(SET_ADDRESS,c, devstruct->ctrl_pipe_nb);
            Host_configure_address(c);
            devstruct->device_addr = c;
            devstruct->device_state = DEVICE_ADDRESSED;
         }
         else
            devstruct->device_state = DEVICE_ERROR; 
    }    
    else if (ds == DEVICE_ADDRESSED) {
        enumerate_task_chrp=host_get_device_std_descriptor(DEVICE_DESCRIPTOR,0,0,devstruct->ctrl_pipe_nb);
         if (enumerate_task_chrp)
         {
             if (Host_notify_new_device((void*)enumerate_task_chrp,devstruct))
                devstruct->device_state = DEVICE_CONFIGURED;
             else 
                devstruct->device_state = DEVICE_ERROR; 
         }
         else // Can not get device descriptor
            devstruct->device_state = DEVICE_ERROR; 
    }
    else if (ds==DEVICE_CONFIGURED) {
        enumerate_task_chrp=host_get_device_std_descriptor(CONFIGURATION_DESCRIPTOR,0,0,devstruct->ctrl_pipe_nb);
        if (enumerate_task_chrp)
        {
            signed char acceptedConfiguration = Host_notify_new_configuration((void*)enumerate_task_chrp,devstruct);
            
            if(acceptedConfiguration<0)
            {
               devstruct->device_state = DEVICE_ERROR;
               return;
            }
            
            if (host_usb_device_set(SET_CONFIGURATION,acceptedConfiguration,devstruct->ctrl_pipe_nb)
                ==CONTROL_GOOD)  // Send Set_configuration
            {
                devstruct->device_state = DEVICE_READY;
                Host_enable_device_disconnection_interrupt();
                // If user host application requires SOF interrupt event
                // Keep SOF interrupt enable otherwize, disable this interrupt
                #if (HOST_CONTINUOUS_SOF_INTERRUPT==DISABLE)
                     Host_disable_sof_interrupt();
                #endif
                
                Enable_interrupt();
                
                Host_device_enumerated_action(devstruct);
            }
            else
                devstruct->device_state = DEVICE_ERROR;  
        }
        else
            devstruct->device_state = DEVICE_ERROR;  
    }
    else if (ds==DEVICE_ERROR) {    // TODO !!!!
      #if (HOST_ERROR_RESTART==ENABLE)
         devstruct->device_state=DEVICE_UNATTACHED;
      #endif
         Host_device_error_action(devstruct);
    }
    else if (ds==DEVICE_SUSPENDED) { // not supported
    }
    else if (ds==DEVICE_WAIT_RESUME) { // not supported
    }
    else if (ds==DEVICE_DISCONNECTED) {
         //------------------------------------------------------
         //   DEVICE_DISCONNECTED state
         //
         //   - Device disconnection has been detected
         //   - Run scheduler in this state at least two times to get sure event is detected by all host application tasks
         //   - Go to DEVICE_DISCONNECTED_ACK state before DEVICE_UNATTACHED, to get sure scheduler calls all app tasks...
         //
         Host_device_disconnection_action(devstruct);
         devstruct->device_state = DEVICE_DISCONNECTED_ACK;
         free_pipes|= (1<<devstruct->ctrl_pipe_nb);
         for (c=0;c<MAX_SUPPORTED_HOST_EP;c++)
         {
             if (devstruct->ep_pipes[c]!=0xff)
             {
                free_pipes|= (1<<devstruct->ep_pipes[c]);
                devstruct->ep_pipes[c]=0xff;
             }
         }     
         free_addresses|= (1<<(devstruct->device_addr-1));
         devstruct->device_addr=0;
    }
    else if (ds==DEVICE_DISCONNECTED_ACK) {
         //------------------------------------------------------
         //   DEVICE_DISCONNECTED_ACK state
         //
         //   - Device disconnection has been detected and managed bu applicatives tasks
         //   - Go to DEVICE_UNATTACHED state
         //
         S_Host_usb_device* devsp = devstruct;
         devstruct->device_state = DEVICE_UNATTACHED;
         do // unlink devstruct.
         {
             if (devstruct == &Host_main_device)
             {
                 free_addresses=0xff;
                 free_pipes=0xff;
                 break; // actually do NOT unlink.
             }
             if (devsp->next == devsp)
             {
                 // error condition..
                 devstruct=&Host_main_device;
                 free_addresses=0xff;
                 free_pipes=0xff;
                 break; // actually do NOT unlink.
             }
             if (devsp->next == devstruct) // update next ptr
             {
                 devsp->next = devstruct->next;
                 devstruct->next=NULL;
                 break;
             }
             devsp = devsp->next;
             
         } while(1);
         
    }
    else {
         //------------------------------------------------------
         //   default state
         //
         //   - Default case: ERROR
         //   - Goto no device state
         //
         devstruct->device_state = DEVICE_UNATTACHED;
   }             
}

// pipe_nb is pipe to connect to.. should be >=1 and unique for each ep.
char host_configure_endpoint(S_usb_endpoint_descriptor * ep_desc, char pipe_nb)
{
         if (pipe_nb==0xff) 
         {
             U8 i;
             for (i=0;i<8;i++)
             {
                 if (free_pipes & (1<<i))
                 {
                     free_pipes &= ~(1<<i);
                     pipe_nb = i;
                     break;
                 }
             }
         }
         else
         {
            free_pipes &= ~(1<<pipe_nb);
         }
         if (pipe_nb==0xff) return 0;

    
        // Select the new physical pipe to configure and get ride of any previous configuration for this physical pipe
         Host_select_pipe(pipe_nb);
         Host_disable_pipe();
         Usb_unallocate_memory();
         Host_enable_pipe();

         //Host_select_pipe(num),              
         
        Host_enable_pipe();
        UPCFG0X = Host_build_pipe_config0(ep_desc->bmAttributes, Get_pipe_token(ep_desc->bEndpointAddress), (ep_desc->bEndpointAddress & MSK_EP_DIR));
        UPCFG1X = Host_build_pipe_config1(host_determine_pipe_size(ep_desc->wMaxPacketSize), ONE_BANK);
        Host_allocate_memory();
        if (! (Is_pipe_configured()))
            return 0xff;
         Host_set_interrupt_frequency(ep_desc->bInterval);
         return pipe_nb;
         
}

//! host_disable_all_pipe.
//!
//!  This function disable all pipes for the host controller
//!  Usefull to execute upon device disconnection.
//!
//! @return none.
//!
void host_disable_all_pipe(void)
{
U8 i;
   for (i=0;i<7;i++)
   {
      Host_reset_pipe(i);
      Host_select_pipe(i);
      Host_unallocate_memory();
      Host_disable_pipe();
   }
}

//! host_determine_pipe_size.
//!
//!  This function returns the size configuration register value according
//!  to the endpint size detected inthe device enumeration process.
//!
//! @return pipe size register value.
//!
U8 host_determine_pipe_size(U16 size)
{
        if(size <= 8  ) {return (SIZE_8   );}
   else if(size <= 16 ) {return (SIZE_16  );}
   else if(size <= 32 ) {return (SIZE_32  );}
   else if(size <= 64 ) {return (SIZE_64  );}
   else if(size <= 128) {return (SIZE_128 );}
   else if(size <= 256) {return (SIZE_256 );}
   else if(size <= 512) {return (SIZE_512 );}
   else                 {return (SIZE_1024);}

}


// ---------- HOST events management -----------------------------------
//! The following USB HOST events are taken in charge:
//! - Device connection
//! - Device Disconnection
//! - Start Of Frame
//! - ID pin change
//! - SOF (or Keep alive in low speed) sent
//! - Wake up on USB line detected
//!

#ifdef AVRGCC
 ISR(USB_GEN_vect)
#else
#pragma vector = USB_General_vect
__interrupt void usb_general_interrupt()
#endif
{
   #if (USB_HOST_PIPE_INTERRUPT_TRANSFER == ENABLE)
//   U8 i;
//   U8 save_pipe_nb;
   #endif
  // - The device has been disconnected
   if(Is_device_disconnection() && Is_host_device_disconnection_interrupt_enabled())
   {
      host_disable_all_pipe();
      Host_ack_device_disconnection();
      Host_main_device.device_state=DEVICE_DISCONNECTED;
      Usb_send_event(EVT_HOST_DISCONNECTION);
   }
  // - Device connection
   if(Is_device_connection() && Is_host_device_connection_interrupt_enabled())
   {
      Host_ack_device_connection();
      host_disable_all_pipe();
      Host_device_connection_action();
   }
  // - Host Start of frame has been sent
   if (Is_host_sof() && Is_host_sof_interrupt_enabled())
   {
      Host_ack_sof();
      Usb_send_event(EVT_HOST_SOF);
      private_sof_counter++;

      // delay timeout management for interrupt tranfer mode in host mode
      #if ((USB_HOST_PIPE_INTERRUPT_TRANSFER==ENABLE) && (TIMEOUT_DELAY_ENABLE==ENABLE))
      if (private_sof_counter>=250)   // Count 1/4 sec
      {
          private_sof_counter=0;
          
         for(i=0;i<MAX_EP_NB;i++)
         {
            if(it_pipe_str[i].enable==ENABLE)
            {
               save_pipe_nb=Host_get_selected_pipe();
               Host_select_pipe(i);
               if((++it_pipe_str[i].timeout>TIMEOUT_DELAY) && (Host_get_pipe_type()!=TYPE_INTERRUPT))
               {
                  it_pipe_str[i].enable=DISABLE;
                  it_pipe_str[i].status=PIPE_DELAY_TIMEOUT;
                  Host_stop_pipe_interrupt(i);
                  if (is_any_interrupt_pipe_active()==FALSE)    // If no more transfer is armed
                  {
                     if (g_sav_int_sof_enable==FALSE)
                     {
                        Host_disable_sof_interrupt();
                     }
                  }
                  it_pipe_str[i].handle(PIPE_DELAY_TIMEOUT,it_pipe_str[i].nb_byte_processed);
               }
               Host_select_pipe(save_pipe_nb);
            }

         }
      }
      #endif  // (USB_HOST_PIPE_INTERRUPT_TRANSFER==ENABLE) && (TIMEOUT_DELAY_ENABLE==ENABLE))
      Host_sof_action();
   }
  // - Host Wake-up has been received
   if (Is_host_hwup() && Is_host_hwup_interrupt_enabled())
   {
      Host_disable_hwup_interrupt();  // Wake up interrupt should be disable host is now wake up !
      // CAUTION HWUP can be cleared only when USB clock is active (not frozen)!
      Pll_start_auto();               // First Restart the PLL for USB operation
      Wait_pll_ready();               // Get sure pll is lock
      Usb_unfreeze_clock();           // Enable clock on USB interface
      Host_ack_hwup();                // Clear HWUP interrupt flag
      Usb_send_event(EVT_HOST_HWUP);  // Send software event
      Host_hwup_action();             // Map custom action
   }
}
#endif // End HOST FEATURE MODE
