#include "conf_usb.h"
#include "usb_host_transfer.h"
#include "usb_host_hub_drv.h"
#include "usb_avr_registers.h"

#if (USB_HOST_FEATURE == ENABLED)

#define MAX_HUB_PORTS 4

static struct {
    U8 nbPorts;
    U_Usb_hub_characteristics wHubCharacteristics;
    S_Host_usb_device portDeviceMap[MAX_HUB_PORTS];
    S_Host_usb_device * hubDev;

} hubinfo ={0,0,{NULL,NULL,NULL,NULL}};

void Host_hubdevice_connectReset(S_Host_usb_device * dev);
U8 Host_hubdevice_state_attached(S_Host_usb_device * dev);



//U8 * host_usb_standard_request(U8 dir_n_type, U8 bRequest, U16 wValue, U16 wIndex, U8 length)
#define usb_hub_SetHubFeature(f,pipe)      host_usb_standard_request(0x20,SET_FEATURE,f,0,0,pipe)
#define usb_hub_ClearHubFeature(f,pipe)    host_usb_standard_request(0x20,CLEAR_FEATURE,f,0,0,pipe)
#define usb_hub_GetHubStatus(pipe)        host_usb_standard_request(0xa0,GET_STATUS,0,0,4,pipe)

//host_get_device_class_descriptor(HUB_DESCRIPTOR,0,ctrl)

#define usb_hub_SetPortFeature(f,p,pipe)   host_usb_standard_request(0x23,SET_FEATURE,f,p,0,pipe)
#define usb_hub_ClearPortFeature(f,p,pipe) host_usb_standard_request(0x23,CLEAR_FEATURE,f,p,0,pipe)
#define usb_hub_GetPortStatus(p,pipe)      host_usb_standard_request(0xa3,GET_STATUS,0,p,4,pipe)

U8 usb_host_hub_enumerated_action(S_Host_usb_device * hubdev)
{
    S_Usb_hub_descriptor * hubdesc;
    U8 i;
    U8 c;
    
    if (hubdev->misc==NULL) // hub ready and this is first run.
    {
        hubdev->misc=(void*)1;
        hubinfo.hubDev=hubdev;
                
        hubdesc = (void*)host_get_device_class_descriptor(HUB_DESCRIPTOR,0,hubdev->ctrl_pipe_nb);
        if (hubdesc==NULL)
            return 1;
        if (hubdesc->bDescriptorType!=HUB_DESCRIPTOR)
            return 2;
        
        hubinfo.nbPorts = hubdesc->bNbrPorts;
        hubinfo.wHubCharacteristics.w = hubdesc->wHubCharacteristics.w;
        
        for (i=1;i<hubinfo.nbPorts+1;i++)
            usb_hub_ClearPortFeature(HUB_FEATURE_PORT_POWER,i,hubdev->ctrl_pipe_nb);

        for (i=1;i<hubinfo.nbPorts+1;i++)
            usb_hub_SetPortFeature(HUB_FEATURE_PORT_POWER,i,hubdev->ctrl_pipe_nb);

        i = Is_host_sof_interrupt_enabled(); //Save current sof interrupt enable state
        Host_enable_sof_interrupt();
        c=0;
        while (c<hubdesc->bPwrOn2PwrGood*2)               // wait for power good
           if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }// Count Start Of frame
        if (i==FALSE)   {Host_disable_sof_interrupt();}
        
    }
    return 0;
}

U16 hub_task_delay=0;
U8 usb_host_hub_task(S_Host_usb_device * hubdev)
{
    U8 i;
    S_Usb_hub_port_status_and_change * pstatus;
    
    hub_task_delay++;
    if (hub_task_delay<0xf000 || hubinfo.nbPorts==0)
        return 0;
    hub_task_delay=0;
        
    
    for (i=1;i<hubinfo.nbPorts+1;i++)
    {
        pstatus = (void*)usb_hub_GetPortStatus(i,hubdev->ctrl_pipe_nb);
        if (pstatus==NULL)
            return 1;
        
        if (pstatus->wChange.w != 0)
        {
            dprintf("port %d status change: %x, status: %x\r",i,pstatus->wChange,pstatus->wStatus);
            // ack status change
            if (pstatus->wChange.w&0x01)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_CONNECTION,i,hubdev->ctrl_pipe_nb);
                if (pstatus->wStatus.PortConnect)
                {
                    hubinfo.portDeviceMap[i-1].device_state= DEVICE_ATTACHED;
                    hubinfo.portDeviceMap[i-1].device_addr=0;
                    hubinfo.portDeviceMap[i-1].ctrl_pipe_nb=0xff;
                    hubinfo.portDeviceMap[i-1].ep_pipes[0]=0xff;
                    hubinfo.portDeviceMap[i-1].ep_pipes[1]=0xff;
                    hubinfo.portDeviceMap[i-1].bDeviceClass=0;
                    hubinfo.portDeviceMap[i-1].misc = NULL;
                    hubinfo.portDeviceMap[i-1].connectReset = Host_hubdevice_connectReset;
                    hubinfo.portDeviceMap[i-1].handle_state_unattached=NULL;
                    hubinfo.portDeviceMap[i-1].handle_state_attached=Host_hubdevice_state_attached;
                    hubinfo.portDeviceMap[i-1].next=hubdev->next;
                    hubdev->next = &(hubinfo.portDeviceMap[i-1]);
                }
                else
                {
                    hubinfo.portDeviceMap[i-1].device_state=DEVICE_DISCONNECTED;
                }
            }
            if (pstatus->wChange.w&0x02)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_ENABLE,i,hubdev->ctrl_pipe_nb);
//                dprintf("   PORT_ENABLE ack'ed\r");
            }
            if (pstatus->wChange.w&0x04)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_SUSPEND,i,hubdev->ctrl_pipe_nb);
//                dprintf("   PORT_SUSPEND ack'ed\r");
            }
            if (pstatus->wChange.w&0x08)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_OVER_CURRENT,i,hubdev->ctrl_pipe_nb);
//                dprintf("   PORT_OC ack'ed\r");
            }
            if (pstatus->wChange.w&0x10)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_RESET,i,hubdev->ctrl_pipe_nb);
//                dprintf("   PORT_RESET ack'ed\r");
            }
            
        }
    }
    
    return 0;
}


void Host_hubdevice_connectReset(S_Host_usb_device * dev)
{
    U8 curAddr = Read_host_address();
    Host_configure_address(hubinfo.hubDev->device_addr);
    U8 portToReset=0xff;
    U8 i;
    U16 c;
    S_Usb_hub_port_status_and_change * pstatus;
    
    for (i=0;i<MAX_HUB_PORTS;i++)
    {
        if (&(hubinfo.portDeviceMap[i])==dev)
        {
            portToReset=i+1;
            break;
        }
    }
    if (portToReset==0xff)
    {
        dev->device_state=DEVICE_ERROR;
        asm("break");
        return;
    }
    usb_hub_SetPortFeature(HUB_FEATURE_PORT_RESET,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
    while(1)
    {
        i=0;
        while(++i !=0); // stupid wait;    
        pstatus = (void*)usb_hub_GetPortStatus(portToReset,hubinfo.hubDev->ctrl_pipe_nb);
        if (pstatus->wChange.w&0x10)
        {
            usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_RESET,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
            if (pstatus->wStatus.Reset == 0)
            {
//                dprintf("   END OF PORT_RESET ack'ed\r");
                break;
            }
            else
            {
//                dprintf("   PORT_RESET ack'ed\r");
            }
        }
    }
    
    

    for(c=0;c!=0xff;c++)    // Basic Timeout counter
    {
        pstatus = (void*)usb_hub_GetPortStatus(portToReset,hubinfo.hubDev->ctrl_pipe_nb);
        if (pstatus->wChange.w != 0)
        {
/*            
            dprintf("port %d status change: %x, status: %x\r",i,pstatus->wChange,pstatus->wStatus);
            // ack status change
            if (pstatus->wChange.w&0x01)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_CONNECTION,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
                dprintf("   PORT_CONNECT ack'ed\r");

            }
            if (pstatus->wChange.w&0x02)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_ENABLE,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
                dprintf("   PORT_ENABLE ack'ed\r");
            }
            if (pstatus->wChange.w&0x04)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_SUSPEND,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
                dprintf("   PORT_SUSPEND ack'ed\r");
            }
            if (pstatus->wChange.w&0x08)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_OVER_CURRENT,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
                dprintf("   PORT_OC ack'ed\r");
            }
            if (pstatus->wChange.w&0x10)
            {
                usb_hub_ClearPortFeature(HUB_FEATURE_C_PORT_RESET,portToReset,hubinfo.hubDev->ctrl_pipe_nb);
                dprintf("   PORT_RESET ack'ed\r");
            }
*/            
        }
    }
    
    
    Host_configure_address(curAddr);
}


U8 Host_hubdevice_state_attached(S_Host_usb_device * dev)
{
    U8 retval = DEVICE_ATTACHED;
    U8 i,c;

    i = Is_host_sof_interrupt_enabled(); //Save current sof interrupt enable state
    Host_enable_sof_interrupt();
    c=0;
    while (c<100)               // wait 100ms before USB reset
    {
        if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }// Count Start Of frame
    }

    
/*
       // Reset device status
        Host_enable_sof_interrupt();  // SOF will be detected under interrupt
        c = 0;
        while (c<100)               // wait 100ms before USB reset
        {
           if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }// Count Start Of frame
           if (Is_host_emergency_exit() || Is_usb_bconnection_error_interrupt()) {
                retval=DEVICE_UNATTACHED;
    
                goto device_attached_error;
            }
        }
*/
    
        dev->connectReset(dev);
        
        c = 0;
        while (c<100)               // wait 100ms after USB reset
        {
           if (Is_usb_event(EVT_HOST_SOF)) { Usb_ack_event(EVT_HOST_SOF); c++; }// Count Start Of frame
        }
        retval = DEVICE_POWERED;
        c=0;

     device_attached_error:
     if (i==FALSE)   {Host_disable_sof_interrupt();}

/*    
    // Device connection error, or vbus pb -> Retry the connection process from the begining
     if( Is_usb_bconnection_error_interrupt()||Is_usb_vbus_error_interrupt()||Is_usb_vbus_low())
     {
     }
*/     
     return retval;
}

#endif
