#include <ioavr.h>
#include "compiler.h"
#include "usb.h"
#include "usb_host_drv.h"

#define HUB_DESCRIPTOR 0x29

typedef union
{
    U16 w;
    struct {
        U16 PowerSwitching:2; // 00 - gang, 01 - individual
        U16 Compound:1;
        U16 OverCurrentProtection:2; // 00 - global, 01 - indiv.
        U16 TTThinkTime:2;
        U16 PortIndicators:1; 
        U16 reserved:8;
    };
} U_Usb_hub_characteristics;
typedef struct
{
    U8  bDescLength;
    U8  bDescriptorType;
    U8  bNbrPorts;
    U_Usb_hub_characteristics wHubCharacteristics;
    U8  bPwrOn2PwrGood;
    U8  bHubContrCurrent;
    U8  DeviceRemovable; // might be more bytes if bNbrPorts>7, bit0 reserved
    U8  PortPwrCtrlMask; // might be more bytes if bNbrPorts>7, bit0 reserved
} S_Usb_hub_descriptor;


#define HUB_FEATURE_C_HUB_LOCAL_POWER 0
#define HUB_FEATURE_C_HUB_OVER_CURRENT 1
#define HUB_FEATURE_PORT_CONNECTION 0
#define HUB_FEATURE_PORT_ENABLE 1
#define HUB_FEATURE_PORT_SUSPEND 2
#define HUB_FEATURE_PORT_OVER_CURRENT 3
#define HUB_FEATURE_PORT_RESET 4
#define HUB_FEATURE_PORT_POWER 8
#define HUB_FEATURE_PORT_LOW_SPEED 9
#define HUB_FEATURE_C_PORT_CONNECTION 16
#define HUB_FEATURE_C_PORT_ENABLE 17
#define HUB_FEATURE_C_PORT_SUSPEND 18
#define HUB_FEATURE_C_PORT_OVER_CURRENT 19
#define HUB_FEATURE_C_PORT_RESET 20
#define HUB_FEATURE_PORT_TEST 21
#define HUB_FEATURE_PORT_INDICATOR 22

typedef struct
{
    U16 wStatus;
    U16 wChange;
} S_Usb_hub_status_and_change;

// 11.24.2.7.1
typedef union
{
    U16 w;
    struct {
        U16 PortConnect:1;
        U16 PortEnabled:1;
        U16 Suspend:1;
        U16 OverCurrent:1;
        U16 Reset:1;
        U16 reserved:3;
        U16 PortPower:1;
        U16 LowSpeedDevice:1;
        U16 HighSpeedDevice:1;
        U16 PortTestMode:1;
        U16 PortIndicatorControl:1;
        U16 reserved2:3;
    };
} U_Usb_hub_port_status;
typedef union
{
    U16 w;
    struct {
        U16 PortConnect:1;
        U16 PortEnabled:1;
        U16 Suspend:1;
        U16 OverCurrent:1;
        U16 Reset:1;
        U16 reserved:11;
    };
} U_Usb_hub_port_change;
typedef struct
{
    U_Usb_hub_port_status wStatus;
    U_Usb_hub_port_change wChange;
} S_Usb_hub_port_status_and_change;

typedef struct S_Usb_hub_devicep
{
    S_Host_usb_device dev;
    struct S_Usb_hub_devicep * next;
} S_Usb_hub_device;



U8 usb_host_hub_task(S_Host_usb_device * hubdev);
