#include "compiler.h"

typedef struct
{
   bit enable;
   U16 nb_byte_to_process;
   U16 nb_byte_processed;
   U16 nb_byte_on_going;
   U8 *ptr_buf;
   void(*handle)(U8 status, U16 nb_byte);
   U8 status;
   U8 timeout;
   U16 nak_timeout;
} S_pipe_int;

typedef struct
{
   U8      bmRequestType;        //!< Characteristics of the request
   U8      bRequest;             //!< Specific request
   U16     wValue;               //!< field that varies according to request
   U16     wIndex;               //!< field that varies according to request
   U16     wLength;              //!< Number of bytes to transfer if Data
   U8      uncomplete_read;      //!< 1 = only one read
}  S_usb_setup_data;

#define PIPE_GOOD             0
#define PIPE_DATA_TOGGLE   0x01
#define PIPE_DATA_PID      0x02
#define PIPE_PID           0x04
#define PIPE_TIMEOUT       0x08
#define PIPE_CRC16         0x10
#define PIPE_STALL         0x20
#define PIPE_NAK_TIMEOUT   0x40
#define PIPE_DELAY_TIMEOUT 0x80


#define CONTROL_GOOD             0
#define CONTROL_DATA_TOGGLE   0x01
#define CONTROL_DATA_PID      0x02
#define CONTROL_PID           0x04
#define CONTROL_TIMEOUT       0x08
#define CONTROL_CRC16         0x10
#define CONTROL_STALL         0x20
#define CONTROL_NO_DEVICE     0x40


#define HOST_FALSE                     0
#define HOST_TRUE                      1

   //! Stop all interrupt attached to a pipe
   #define Host_stop_pipe_interrupt(i) (\
         Host_disable_transmit_interrupt(), \
         Host_disable_receive_interrupt(), \
         Host_disable_stall_interrupt(), \
         Host_disable_error_interrupt(), \
         Host_disable_nak_interrupt(), \
         Host_reset_pipe(i))

U8 host_send_control(U8* data_pointer, U8 ctrl_pipe_nb);
U8 host_send_data_interrupt(U8 pipe, U16 nb_data, U8 *buf, void(*handle)(U8 status, U16 nb_byte));
U8 host_get_data_interrupt(U8 pipe, U16 nb_data, U8 *buf,void(*handle)(U8 status, U16 nb_byte));
U8 host_get_data(U8 pipe, U16 *nb_data, U8 *buf);
U8 host_send_data(U8 pipe, U16 nb_data, U8 *buf);

U8 * host_get_device_std_descriptor(U8 desctype,U8 wValue, U8 wIndex,U8 ctrl_pipe);
U8 * host_get_device_class_descriptor(U8 desctype,U8 wValue, U8 ctrl_pipe) ;
U8 * host_get_interface_std_descriptor(U8 desctype,U8 idx, U8 ifnb,U8 ctrl_pipe);

U8 * host_usb_standard_request(U8 dir_n_type, U8 bRequest, U16 wValue, U16 wIndex, U8 length,U8 ctrl_pipe);

U8 host_usb_device_set(U8 req, U8 value, U8 ctrl_pipe);
U8 host_usb_device_setEx(U8 req, U8 value, U8 indx, U8 ctrl_pipe);

//U8 host_set_address(U8 addr,U8 ctrl_pipe);
